package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.ConceptFacade;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptUsualVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 概念表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2019-05-06
 */
@RestController
@RequestMapping("/concept")
@Api(value = "常用标签API", tags = { "知识库标准化-常用标签API" })
@SuppressWarnings("unchecked")
public class ConceptController {
    @Autowired
    ConceptFacade conceptFacade;

    @ApiOperation(value = "知识库标准化-常用标签——根据科室获取常用标签[by:wangyu]",
            notes = "deptCode：科室id<br>" +
                    "age：年龄<br>" +
                    "sexType: 性别<br>" +
                    "type：类型(1：症状 3：其他史 4：查体，5：化验 6：辅检 7：诊断)，必填<br>")
    @PostMapping("/getConceptUsual")
    @SysLogger("getConceptUsual")
    public RespDTO<List<ConceptBaseDTO>> getConceptUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO) {
        List<ConceptBaseDTO> data = conceptFacade.getConceptUsual(conceptUsualVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 根据名称和词性获取概念Map
     *
     * @param conceptSearchVO 搜索参数
     * @return 术语和术语id Map
     */
    @ApiOperation(value = "知识库标准化-根据名称和词性获取概念列表[by:gaodm]",
            notes = "nameList: 术语列表<br>" +
                    "type：类型(1：症状 4：查体，5：化验 6：辅检 7：诊断)，必填<br>")
    @PostMapping("/getConceptMap")
    @SysLogger("getConceptMap")
    public RespDTO<Map<String, Long>> getConceptMap(@RequestBody ConceptSearchVO conceptSearchVO){
        return RespDTO.onSuc(conceptFacade.getConceptMap(conceptSearchVO));
    }
}
